C	*********************** PMEHISTO MAINLINE ***************************
C
C
	PROGRAM PMEHISTO
C
C
C	THE PMEHISTO PROGRAM TAKES A BUCKET (.PME) FILE AND POSSIBLY A TIME-
C	SAMPLE (.PMS) FILE AS INPUT AND FROM THIS PRODUCES A HISTOGRAM OF HOW
C	MUCH TIME WAS SPENT IN EACH OF THE BUCKETS SPECIFIED BY THE BUCKET
C	FILE.
C
C
C			WRITTEN BY BERT BEANDER, MARCH, 1979.
C
C
C
C	***DATA DEFINITIONS
	INCLUDE 'PMEHISCOM.FOR/LIST'
	INCLUDE 'PMECOMDEF.FOR/LIST'
C
	PARAMETER LINELEN = 50, MAXLINES = 50, TERMLINES = 15
	CHARACTER*50 ASTERISKS
	CHARACTER*16 CURNAME
	CHARACTER*1 ANSWER
	LOGICAL PRINTFLG, ABSFLG
	REAL SCALE, ONEGEAR, PERCENT
C
C	***SPECIAL EQUIVALENCING SO WE CAN COPY A CHARACTER STRING FROM THE
C	***INTEGER BUFFER TO A CHARACTER VARIABLE.
	CHARACTER*60 CHRNAME
	BYTE BYTNAME(60)
	DIMENSION INTNAME(15)
	EQUIVALENCE (CHRNAME,BYTNAME(1)), (BYTNAME(1),INTNAME(1))
C
C
C	*********************** GET THE INPUT DATA *************************
C
C
C	***ASK THE USER WHETHER HE HAS CLOCK-SAMPLED OR TRACE-SAMPLED INPUT
   10	PRINT 20
   20	FORMAT(' Do you have Clock-sampled or Trace data? ',
     1		'(Clock or Trace): ',$)
	READ(KBD,30) ANSWER
   30	FORMAT(A1)
	IF(ANSWER .EQ. 'C' .OR. ANSWER .EQ. 'c') THEN
	    PRINT 40
   40	    FORMAT(' --- Clock-sampled data ---'/)
	    CALL CLOCKSAMPLE
	    GO TO 200
	ENDIF
	IF(ANSWER .NE. 'T' .AND. ANSWER .NE. 't') GO TO 10
	PRINT 50
   50	FORMAT(' --- Trace-sampled data ---'/)
C
C	***TRACE DATA--ASK THE USER WHETHER HE WANTS TO SPECIFY FILE NAMES
   70	PRINT 80
   80	FORMAT(' Do you wish to specify file names? (Yes or No): ',$)
	READ(KBD,30) ANSWER
	IF(ANSWER .EQ. 'Y' .OR. ANSWER .EQ. 'y') GO TO 100
	IF(ANSWER .NE. 'N' .AND. ANSWER .NE. 'n') GO TO 70
C
C	***HE DOES NOT WANT TO SPECIFY FILE NAMES--GIVE HIM DEFAULT NAMES
	OPEN(UNIT=10, NAME='PMEFILE.HIS', TYPE='NEW')
	GO TO 150
C
C	***HE WANTS TO SPECIFY FILE NAMES--ASK HIM WHAT THEY ARE
  100	PRINT 110
  110	FORMAT(' Enter name of input bucket (.PME) file: ',$)
	READ(KBD,120) INFNAME
  120	FORMAT(A40)
	CALL PME_INAME(INFNAME)
	PRINT 130
  130	FORMAT(' Enter name of output histogram (.HIS) file: ',$)
	READ(KBD,120) HISNAME
	IF(HISNAME .EQ. ' ') HISNAME = 'PMEFILE.HIS'
	OPEN(UNIT=10, NAME=HISNAME, TYPE='NEW')
C
C	***READ THE BUCKET FILE INTO OUR BUFFER (WITHOUT CLEARING ANY COUNTS)
  150	CALL PME_OPEN(BUFFER, MAXBUF+1, .FALSE.)
	IF(BUFFER(0) .EQ. 0) STOP '--> OPEN ERROR IN PMEHISTO'
C
C
C	********* WE HAVE THE BUCKET FILE IN BUFFER--NOW PROCESS IT *********
C
C
C	***GET THE ABSOLUTE VS RELATIVE ADDRESSES FLAG FROM THE HEADER RECORD
  200	ABSFLG = IAND(BUFFER(HDRFLAGS), HDRFLG_ABS) .NE. 0
C
C	***PICK UP THE NUMBER OF DISTINCT BUCKETS AND INITIALIZE ALL BUCKET
C	***COUNTS TO ZEROES.
	NBKTS = BUFFER(HDRNUMBKTS)
	IF(NBKTS .LE. 0 .OR. NBKTS .GT. MAXBKTS) THEN
	    PRINT 210
  210	    FORMAT('0*** Error:  Number of buckets in bucket file is',
     1		' too large or too small ***')
	    STOP
	ENDIF
	DO 230 I=1,NBKTS
	BKTLOW(I) = 1
	BKTHIGH(I) = 0
	BKTCOUNT(I) = 0
  230	CONTINUE
C
C	***PICK UP ALL BUCKET GROUP ADDRESS RANGES AND GROUP PRINT NAMES
	GRPPTR = BUFFER(HDRBGRPTR)/4
	NGRPS = 0
  250	IF(BUFFER(GRPPTR+BUCKETSIZE) .EQ. 0) GO TO 270
	IF(NGRPS .EQ. MAXGRPS) STOP '--> ERROR 1 PMEHISTO'
	NGRPS = NGRPS + 1
	GRPLOW(NGRPS) = BUFFER(GRPPTR+GRPLOWBND)
	GRPHIGH(NGRPS) = BUFFER(GRPPTR+GRPHIBND)
	DO 260 I=1,15
  260	INTNAME(I) = BUFFER(GRPPTR+GRPNAME+I-1)
	GROUPNAME(NGRPS) = CHRNAME
	GRPPTR = GRPPTR + GRPRECSIZ
	GO TO 250
  270	IF(NGRPS .EQ. 0) STOP '--> ERROR 2 PMEHISTO'
C
C	***NOW PICK UP ALL BUCKET COUNTS AND ADDRESS RANGES FROM THE BUCKET FILE
	ILO = BUFFER(HDRBRECPTR)/4 + BKTRECSIZ
	IHI = BUFFER(HDRNAMPTR)/4 - BKTRECSIZ
	DO 300 I=ILO,IHI,BKTRECSIZ
	IBKT = BUFFER(I + BUCKETNUM)
	IF(IBKT .LE. 0 .OR. IBKT .GT. NBKTS) STOP '--> ERROR 3 PMEHISTO'
	BKTLOW(IBKT) = BUFFER(I + BKTLOWBND)
	BKTHIGH(IBKT) = BUFFER(I + BKTHIBND)
	BKTCOUNT(IBKT) = BKTCOUNT(IBKT) + BUFFER(I + COUNT)
  300	CONTINUE
C
C	***COMPUTE THE TOTAL OF ALL REFERENCE COUNTS IN THE WHOLE BUCKET FILE.
C	***ALSO GET THE LARGEST SINGLE COUNT FOR SCALING THE HISTOGRAM.
	NTOTAL = BUFFER(HDRDEFCNTR)
	MAXCOUNT = 1
	DO 320 IBKT=1,NBKTS
	IF(BKTLOW(IBKT) .GT. BKTHIGH(IBKT)) STOP '--> ERROR 4 PMEHISTO'
	NTOTAL = NTOTAL + BKTCOUNT(IBKT)
	MAXCOUNT = MAX0(MAXCOUNT, BKTCOUNT(IBKT))
  320	CONTINUE
	SCALE = FLOAT(LINELEN)/FLOAT(MAXCOUNT)
	ONEGEAR = 1.0/SCALE
C
C	***MAKE SURE THERE ARE COUNTS IN THE BUCKET FILE
	IF(NTOTAL .EQ. 0) THEN
	    PRINT 330
  330	    FORMAT('0*** Error:  Total PC count in file is zero--',
     1		'No histogram produced ***')
	    STOP
	ENDIF
C
C	***PICK UP EACH BUCKET'S SYMBOLIC NAME, BASE ADDRESS, AND GROUP INDEX
	NAMPTR = BUFFER(HDRNAMPTR)/4
	DO 370 IBKT=1,NBKTS
	BKTBASE(IBKT) = BUFFER(NAMPTR)
	NAMPTR = NAMPTR + 1
	CHRNAME = ' '
	DO 340 I = 1, NAMRECSIZ-1
	INTNAME(I) = BUFFER(NAMPTR)
	NAMPTR = NAMPTR + 1
  340	CONTINUE
	BKTNAME(IBKT) = CHRNAME
	DO 350 I=1,NGRPS
	IF(BKTLOW(IBKT) .GE. GRPLOW(I) .AND.
     1		BKTLOW(IBKT) .LE. GRPHIGH(I)) GO TO 360
  350	CONTINUE
	STOP '--> ERROR 5 PMEHISTO'
  360	BKTGRPPTR(IBKT) = I
  370	CONTINUE
C
C
C	******************* PRINT THE ACTUAL HISTOGRAM **********************
C
C
C	***LOOP THROUGH ALL THE BUCKETS TO PRINT OUT THE HISTOGRAM
	PAGECNT = 0
	LINECNT = MAXLINES
	CURGROUP = 0
	CURNAME = ' '
	DO 450 IBKT=1,NBKTS
C
C	***SUPPRESS THE BUCKET NAME FOR ALL BUCKETS EXCEPT THE FIRST IN EACH
C	***PROGRAM UNIT.
	IF(BKTNAME(IBKT) .EQ. CURNAME) THEN
	    PRINTFLG = .FALSE.
	ELSE
	    CURNAME = BKTNAME(IBKT)
	    PRINTFLG = .TRUE.
	ENDIF
C
C	***TAKE CARE OF PAGE HEADERS AND TRAILERS WHEN CROSSING A PAGE BOUNDARY
	IF(LINECNT .GE. MAXLINES .OR. BKTGRPPTR(IBKT) .NE. CURGROUP
     1		.OR. (LINECNT .EQ. MAXLINES-1 .AND. PRINTFLG)) THEN
	    CURGROUP = BKTGRPPTR(IBKT)
	    IF(PAGECNT .NE. 0) THEN
		IF(LINECNT .LT. MAXLINES) THEN
		    DO 390 I = 1, MAXLINES-LINECNT
  390		    WRITE(10,420) ' '
		ENDIF
		WRITE(10,400) ONEGEAR
  400		FORMAT(20X,'+',10('----+')//
     1			28X,'Scaling:',F9.1,' counts/asterisk')
	    ENDIF
	    PAGECNT = PAGECNT + 1
	    WRITE(10,410) PAGECNT, GROUPNAME(CURGROUP)
  410	    FORMAT('1',10X,'Performance Measurement and Evaluation',
     1		' Histogram',12X,'Page',I3//15X,A60//20X,'+',10('----+'))
	    LINECNT = 0
	    PRINTFLG = .TRUE.
	ENDIF
C
C	***PRINT OUT THE BUCKET NAME UNLESS IT IS SUPPRESSED
	IF(PRINTFLG) THEN
	    WRITE(10,420) CURNAME
  420	    FORMAT(1X,A16,3X,'|')
	    LINECNT = LINECNT + 1
	ENDIF
C
C	***PRINT THE ADDRESS RANGE, THE HISTOGRAM BAR, AND THE PERCENTAGE OF
C	***TOTAL COUNTS REPRESENTED BY THIS BUCKET.
	IF(ABSFLG) THEN
	    LOW = BKTLOW(IBKT)
	    HIGH = BKTHIGH(IBKT)
	ELSE
	    LOW = BKTLOW(IBKT) - BKTBASE(IBKT)
	    HIGH = BKTHIGH(IBKT) - BKTBASE(IBKT)
	ENDIF
	NGEARS = NINT(FLOAT(BKTCOUNT(IBKT))*SCALE)
	ASTERISKS = ' '
	IF(NGEARS .GT. 0) THEN
	    DO 430 I=1,NGEARS
  430	    ASTERISKS(I:I) = '*'
	ENDIF
	PERCENT = 100.0*FLOAT(BKTCOUNT(IBKT))/FLOAT(NTOTAL)
	WRITE(10,440) LOW, HIGH, ASTERISKS, PERCENT
  440	FORMAT(1X,Z8,' -',Z8,' |',A50,F6.1,'%')
	LINECNT = LINECNT + 1
  450	CONTINUE
C
C	***THE HISTOGRAM IS DONE--APPEND A PAGE TRAILER TO THE LAST PAGE
	IF(LINECNT .LT. MAXLINES) THEN
	    DO 460 I = 1, MAXLINES-LINECNT
  460	    WRITE(10,420) ' '
	ENDIF
	WRITE(10,400) ONEGEAR
C
C	***THEN PRINT SOME SUMMARY STATISTICS
	PERCENT = 100.0*FLOAT(NTOTAL-BUFFER(HDRDEFCNTR))/FLOAT(NTOTAL)
	WRITE(10,470) NTOTAL, PERCENT, BUFFER(HDRNUMRUNS), NBKTS
  470	FORMAT('1   *** Summary Statistics ***'//
     1		'0Total number of PC samples:',I8/
     2		' Percent in defined buckets:',F7.1,'%'/
     3		' Number of sample collections:',I6/
     4		' Number of distinct buckets:',I8/)
	CLOSE(UNIT=10)
C
C	***TELL THE USER (ON HIS TERMINAL) THAT WE ARE DONE WITH THE FILE.
	PRINT 480
  480	FORMAT('0*** Histogram file generated ***'/)
C
C
C	************* GENERATE HISTOGRAM DISPLAY ON TERMINAL ****************
C
C
C	***ANNOUNCE THAT WE ARE STARTING THE HISTOGRAM DISPLAY ON THE USER'S
C	***TERMINAL AND START UP A LOOP THROUGH ALL THE BUCKETS.
  500	PRINT 510
  510	FORMAT('0------Start of histogram display on terminal------'/
     1		'0Enter H if you need help.'/)
	PAGECNT = 0
	LINECNT = TERMLINES
	CURGROUP = 0
	CURNAME = ' '
	DO 580 IBKT=1,NBKTS
C
C	***SUPPRESS THE BUCKET NAME FOR ALL BUCKETS EXCEPT THE FIRST IN EACH
C	***PROGRAM UNIT.
	IF(BKTNAME(IBKT) .EQ. CURNAME) THEN
	    PRINTFLG = .FALSE.
	ELSE
	    CURNAME = BKTNAME(IBKT)
	    PRINTFLG = .TRUE.
	ENDIF
C
C	***TAKE CARE OF PAGE HEADERS AND TRAILERS WHEN CROSSING A PAGE BOUNDARY.
C	***ALSO WAIT FOR THE USER TO ENTER THE NEXT COMMAND SO HE GETS A CHANCE
C	***TO STUDY THE PRESENT PAGE.
	IF(LINECNT .GE. TERMLINES .OR. BKTGRPPTR(IBKT) .NE. CURGROUP
     1		.OR. (LINECNT .EQ. TERMLINES-1 .AND. PRINTFLG)) THEN
	    CURGROUP = BKTGRPPTR(IBKT)
	    IF(PAGECNT .NE. 0) THEN
		IF(LINECNT .LT. TERMLINES) THEN
		    DO 520 I = 1, TERMLINES-LINECNT
  520		    PRINT 420, ' '
		ENDIF
		PRINT 400, ONEGEAR
	    ENDIF
  530	    PRINT 540
  540	    FORMAT(' Enter command: ',$)
	    READ(KBD,30,END=650) ANSWER
	    IF(ANSWER .EQ. 'R' .OR. ANSWER .EQ. 'r') GO TO 500
	    IF(ANSWER .EQ. 'S' .OR. ANSWER .EQ. 's') GO TO 620
	    IF(ANSWER .EQ. 'K' .OR. ANSWER .EQ. 'k') GO TO 650
	    IF(ANSWER .EQ. 'H' .OR. ANSWER .EQ. 'h') THEN
		PRINT 610
		GO TO 530
	    ENDIF
	    IF(ANSWER .NE. ' ') GO TO 530
	    PAGECNT = PAGECNT + 1
	    PRINT 410, PAGECNT, GROUPNAME(CURGROUP)
	    LINECNT = 0
	    PRINTFLG = .TRUE.
	ENDIF
C
C	***PRINT OUT THE BUCKET NAME UNLESS IT IS SUPPRESSED
	IF(PRINTFLG) THEN
	    PRINT 420, CURNAME
	    LINECNT = LINECNT + 1
	ENDIF
C
C	***PRINT THE ADDRESS RANGE, THE HISTOGRAM BAR, AND THE PERCENTAGE OF
C	***TOTAL COUNTS REPRESENTED BY THIS BUCKET.
	IF(ABSFLG) THEN
	    LOW = BKTLOW(IBKT)
	    HIGH = BKTHIGH(IBKT)
	ELSE
	    LOW = BKTLOW(IBKT) - BKTBASE(IBKT)
	    HIGH = BKTHIGH(IBKT) - BKTBASE(IBKT)
	ENDIF
	NGEARS = NINT(FLOAT(BKTCOUNT(IBKT))*SCALE)
	ASTERISKS = ' '
	IF(NGEARS .GT. 0) THEN
	    DO 570 I=1,NGEARS
  570	    ASTERISKS(I:I) = '*'
	ENDIF
	PERCENT = 100.0*FLOAT(BKTCOUNT(IBKT))/FLOAT(NTOTAL)
	PRINT 440, LOW, HIGH, ASTERISKS, PERCENT
	LINECNT = LINECNT + 1
  580	CONTINUE
C
C	***THE HISTOGRAM IS DONE--APPEND A PAGE TRAILER TO THE LAST PAGE
C	***AND WAIT FOR THE USER'S RESPONSE.
	IF(LINECNT .LT. TERMLINES) THEN
	    DO 590 I = 1, TERMLINES-LINECNT
  590	    PRINT 420, ' '
	ENDIF
	PRINT 400, ONEGEAR
  600	PRINT 540
	READ(KBD,30,END=650) ANSWER
	IF(ANSWER .EQ. 'R' .OR. ANSWER .EQ. 'r') GO TO 500
	IF(ANSWER .EQ. 'S' .OR. ANSWER .EQ. 's') GO TO 620
	IF(ANSWER .EQ. 'K' .OR. ANSWER .EQ. 'k') GO TO 650
	IF(ANSWER .EQ. 'H' .OR. ANSWER .EQ. 'h') THEN
	    PRINT 610
  610	    FORMAT('0The valid commands for the terminal display are:'/
     1		'     Carriage Return -- Display next histogram page'/
     2		'     R -- Restart histogram display from beginning'/
     3		'     S -- Skip to the summary statistics display'/
     4		'     K -- Kill the display and exit program'/
     5		'     H -- Help: print this display command list'///)
	    GO TO 600
	ENDIF
	IF(ANSWER .NE. ' ') GO TO 600
C
C	***THEN PRINT THE SUMMARY STATISTICS
  620	PERCENT = 100.0*FLOAT(NTOTAL-BUFFER(HDRDEFCNTR))/FLOAT(NTOTAL)
	PRINT 470, NTOTAL, PERCENT, BUFFER(HDRNUMRUNS), NBKTS
	PRINT 630
  630	FORMAT('0*** Histogram completed ***'/)
	GO TO 500
C
C	***ALL DONE--PRINT A BYE-BYE MESSAGE AND EXIT
  650	PRINT 660
  660	FORMAT('0*** Histogram display terminated ***')
	END

	SUBROUTINE CLOCKSAMPLE
C
C
C	SUBROUTINE TO PROCESS THE CLOCK-SAMPLED INPUT FILE (WHICH CONTAINS
C	PROGRAM COUNTER VALUES COLLECTED BY CLOCK-INTERRUPT SAMPLING).  THIS
C	ROUTINE IS CALLED AS FOLLOWS:
C
C				CALL CLOCKSAMPLE
C
C	THE CLOCK SAMPLE FILE IS READ IN AND COMBINED WITH THE APPROPRIATE
C	BUCKET FILE TO PRODUCE A NEW BUCKET FILE.  THE CONTENTS OF THIS NEW
C	BUCKET FILE (WHICH IS LEFT IN THE BUFFER ARRAY) IS THEN PROCESSED
C	BY THE MAINLINE TO PRODUCE THE ACTUAL HISTOGRAM.
C
C
C	***DATA DEFINITIONS
	INCLUDE 'PMEHISCOM.FOR/NOLIST'
	INCLUDE 'PMECOMDEF.FOR/NOLIST'
C
	CHARACTER*1 ANSWER
	DIMENSION HEADER(128), PCBLK(128)
C
C
C	***INITIALIZE A FEW THINGS
	NUMPCS  = 0
	PMSNAME = ' '
	INFNAME = ' '
	OUTFNAME = ' '
	HISNAME = ' '
C
C	***ASK THE USER WHETHER HE WANTS TO SPECIFY INPUT AND OUTPUT FILE NAMES
   10	PRINT 20
   20	FORMAT(' Do you wish to specify file names? (Yes or No): ',$)
	READ(KBD,30) ANSWER
   30	FORMAT(A1)
	IF(ANSWER .EQ. 'Y' .OR. ANSWER .EQ. 'y') GO TO 40
	IF(ANSWER .NE. 'N' .AND. ANSWER .NE. 'n') GO TO 10
C
C	***HE DOES NOT WISH TO SPECIFY FILE NAMES--GIVE HIM THE DEFAULT NAMES
	OPEN(UNIT=2, NAME='PMEFILE.PMS', TYPE='OLD', ERR=220)
	OPEN(UNIT=10, NAME='PMEFILE.HIS', TYPE='NEW')
	GO TO 100
C
C	***HE WANTS TO SPECIFY FILE NAMES--ASK HIM WHAT THEY ARE
   40	PRINT 50
   50	FORMAT(' Enter name of clock sample (.PMS) file: ',$)
	READ(KBD,60) PMSNAME
   60	FORMAT(A40)
	IF(PMSNAME .EQ. ' ') PMSNAME = 'PMEFILE.PMS'
	OPEN(UNIT=2, NAME=PMSNAME, TYPE='OLD', ERR=220)
	PRINT 70
   70	FORMAT(' Enter name of input bucket (.PME) file: ',$)
	READ(KBD,60) INFNAME
	CALL PME_INAME(INFNAME)
	PRINT 80
   80	FORMAT(' Enter name of output bucket (.PME) file: ',$)
	READ(KBD,60) OUTFNAME
	IF(OUTFNAME .EQ. ' ') OUTFNAME = INFNAME
	CALL PME_ONAME(OUTFNAME)
	PRINT 90
   90	FORMAT(' Enter name of output histogram (.HIS) file: ',$)
	READ(KBD,60) HISNAME
	IF(HISNAME .EQ. ' ') HISNAME = 'PMEFILE.HIS'
	OPEN(UNIT=10, NAME=HISNAME, TYPE='NEW')
C
C	***NOW READ IN THE INPUT BUCKET FILE (ALLOW COUNTS TO BE CLEARED)
  100	CALL PME_OPEN(BUFFER, MAXBUF+1, .TRUE.)
	IF(BUFFER(0) .EQ. 0) STOP '--> OPEN ERROR IN CLOCKSAMPLE'
C
C	***READ THE NEXT SAMPLE FILE HEADER RECORD.  (THERE MAY BE MANY SUCH
C	***HEADERS IN ONE SAMPLE FILE BECAUSE THE USER IS ALLOWED TO CONCATE-
C	***NATE SAMPLE FILES TOGETHER AND THEN PROCESSES THE RESULTING FILE.)
  120	READ(2,130,END=180) HEADER
  130	FORMAT(128A4)
C
C	***READ IN THE NEXT BLOCK OF PROGRAM COUNTER VALUES AND TALLY ALL SUCH
C	***VALUES IN THE PROPER BUCKETS.  A ZERO PROGRAM COUNTER VALUE (WHICH
C	***IS OTHERWISE ILLEGAL) IS AN END-OF-SAMPLES MARKER.
  150	READ(2,130,END=200) PCBLK
	DO 160 I=1,128
	IF(PCBLK(I) .EQ. 0) GO TO 120
	NUMPCS = NUMPCS + 1
	CALL PME_COUNTER(PCBLK(I))
  160	CONTINUE
	GO TO 150
C
C	***ALL DONE WITH SAMPLE FILE--CREATE A NEW BUCKET FILE AND WRITE OUT
C	***THE UPDATED BUCKET COUNTS
  180	CALL PME_CLOSE(BUFFER)
	PRINT 190, NUMPCS
  190	FORMAT('0*** Sample file processed--',I7,' samples entered in',
     1		' bucket file ***')
	RETURN
C
C	***INTERNAL ERROR IN THE SAMPLE FILE--GIVE ERROR MESSAGE AND QUIT
  200	PRINT 210
  210	FORMAT('0*** Error in clock sample file--no end-of-sampling',
     1		' marker ***')
	STOP
C
C	***OPEN ERROR ON THE SAMPLE FILE--GIVE ERROOR MESSAGE AND QUIT
  220	PRINT 230
  230	FORMAT('0*** Error:  Clock sample (.PMS) file could not be',
     1		' opened ***')
	STOP
	END

