C	GETSEND.FOR
C
C	THESE ROUTINES PERFORM THE GET AND SEND FUNCTIONS FROM AND TO THE
C	PDP-10, VAX, OR PDP11.
C
C	ROGER LIPSETT	INTERMETRICS	APRIL 27, 1979
C
C	MODIFICATION HISTORY
C
C	TECOINPUT COMMAND ADDED MAY 11, 1979 - J. THOMPSON  (REV 1.1)	
C	MINOR EDITING FOR VAXNET  J. THOMPSON (REV 1.2)
C	ALL QIO READS MODIFIED TO INCLUDE NO ECHOING OF TERMINATORS.
C	JULY 13, 1979   J. THOMPSON (REV 1.53)
C
	SUBROUTINE GETSEND
C
	INCLUDE 'COM.INC'
C
	LOGICAL I, INIT10

C	A COMMAND IS IN PROGRESS
	INCMD = .TRUE.

C	REENABLE THE ^C TRAPS.
	CALL SETCTRLC

C	TEST FOR TECO INPUT.  NO PROTOCOL FOR TECOINPUT.
	IF(MODE.EQ.TECO) GO TO 100 !TECO MUST BE RUNNING

C	WHEN USING THE SNDRCV PROGRAM, THIS DOES THE HANDSHAKING.
	IF (.NOT. INIT10()) GO TO 2000

100	WRITE (6,9904)
9904	FORMAT(' NAME OF THE VAX FILE? '$)
	READ (5,9905) VSIZE,VAXFILE(1:VSIZE)
9905	FORMAT(Q,A<VSIZE>)

	IF (FLOW .EQ. IN) THEN
		I = GET()
	ELSE
		I = SEND()
	ENDIF
	IF (I) GO TO 3000

	TYPE *, 'FILE TRANSMISSION ABORTED.'
	GO TO 2100

2000	TYPE *, 'ERROR INITIATING SNDRCV FILE TRANSFER PROGRAM.'
2100	TYPE *, '	RETURNING TO VIRTUAL TERMINAL LEVEL.'
	TYPE *, '	PLEASE SEND TWO ^C''S TO THE REMOTE TO STOP SNDRCV.'
	GO TO 3100

3000	CALL PRTFINAL
	TYPE 3001, BCOUNT
3001	FORMAT( ' FILE TRANSMISSION COMPLETE, ',I,' BYTES TRANSFERRED.')

3100	CLOSE (UNIT=1)		! SAVE WHATEVER WE TRANSMITTED.
	INCMD = .FALSE.		! NO LONGER IN A COMMAND.
	CALL REENABLE
	RETURN
	END
	LOGICAL FUNCTION GET
C
C	ROUTINE TO TRANSFER A FILE FROM THE REMOTE.  THE MODE OF THE TRANSFER
C	IS STORED IN MODE ( A COMMON VARIABLE ).  THE POSSIBILITIES ARE:
C	1.	ASCII.  A NORMAL ASCII FILE, NO FORTRAN CARRIAGE CONTROL.
C		IN THIS CASE, ASCII OUTPUT TO THE VAX IS DONE TO A FILE WITH
C		CARRIAGECONTROL='LIST'.  MAXIMUM LINE LENGTH
C		IS 200 BYTES (CONTROLLED BY LINE BUFFER ON THE REMOTE SIDE).
C	2.	LISTING.  A LISTING FILE, WITH THE FIRST CHARACTER OF EACH
C		RECORD BEING THE FORTRAN CARRIAGE CONTROL CHARACTER.  THE
C		REMOTE WILL NOT KNOW WHAT IS GOING ON, THE ONLY DIFFERENCE
C		BETWEEN THIS AND MODE ASCII BEING THAT THE FILE IN THIS MODE
C		HAS CARRIAGECONTROL='FORTRAN'.
C	3.	SYSGEN. (-10 ONLY) A UYK-20 SYSGEN FORMAT BINARY FILE.
C		SUCH A FILE IS PACKED TWO 16-BIT WORDS TO A 36-BIT WORD,
C		LEFT JUSTIFIED, ON THE -10.  EACH RECORD HAS A COUNT RECORD
C		PRECEDING IT, WHICH IS NOT TRANSMITTED BY THE -10.  THE BYTE
C		ORDER OF EACH WORD IS REVERSED IN THE TRANSFER.  BYTES ARE
C		SPLIT UP INTO NIBBLES AND ORED WITH "100 TO PREVENT CONTROL
C		CHARACTER PROBLEMS IN THE READPBLK.
C
	INCLUDE 'COM.INC/NOLIST'
C
	BYTE ACK
	INTEGER GET_CHECKSUM

	IF (MODE .EQ. ASCII) THEN
		OPEN (UNIT=1, NAME=VAXFILE(1:VSIZE),
	1		CARRIAGECONTROL='LIST')
	ELSEIF (MODE .EQ. LISTING) THEN
		OPEN (UNIT=1, NAME=VAXFILE(1:VSIZE),
	1		CARRIAGECONTROL='FORTRAN')
	ELSE ! MODE .EQ. SYSGEN
		BUFIDX = 0		! INITIALIZE THE BUFFER POINTER.
		OPEN (UNIT=1, NAME=VAXFILE(1:VSIZE), FORM='UNFORMATTED',
	1		RECORDTYPE='VARIABLE', RECORDSIZE=UYK_REC_SIZE)
	ENDIF
C	SYNCHRONIZE WITH REMOTE CPU
	XBUFFER(1)=1HZ !SYNCH CHARACTER
5	IRCODE=WRTRD(1)
	IF(IRCODE.NE.%LOC(SS$_NORMAL)) CALL QIOERROR(11,IRCODE)
	NBYTES=RIOSB(2)
	CALL STRIPLF(IDX,RBUFFER,NBYTES)
	IF(NBYTES.EQ.1.AND.RBUFFER(IDX).EQ.1HY) THEN
		GO TO 6 !OK
	ELSE
		GO TO 5 !TRY AGAIN
	ENDIF
C
C	LOOP READING FROM THE REMOTE CPU.
C
6	LINNUM = 0		! INITIALIZE LINE COUNTER.
	BCOUNT = 0		! AND BYTE COUNT.
	GET = .FALSE.		! DEFAULT TO ABORT RETURN.
10	IF (.NOT. INCMD) RETURN
	IRCODE = SYS$QIOW(,%VAL(RCHANIN),%VAL(%LOC(IO$_READPBLK)
	1		+%LOC(IO$M_TRMNOECHO)),
	1		RIOSB,,,RBUFFER,%VAL(BUFSIZE),,,,)
	IF (IRCODE .EQ. %LOC(SS$_CANCEL)) RETURN
	IF (IRCODE .NE. %LOC(SS$_NORMAL)) CALL QIOERROR(9,IRCODE)
	NBYTES = RIOSB(2)		! GET NUMBER OF BYTES READ.
	CALL STRIPLF(IDX, RBUFFER, NBYTES) ! STRIP A LEADING LF IF ANY.
	ACK = 1HN			! INITIALIZE ACK TO "N" (RESEND)
	IF (NBYTES .LT. 3) GO TO 10	! SOME SYSTEMS MAY SEND CR FIRST
	IF (NBYTES .LT. 7) GO TO 30	! IF < 7 BYTES, ERROR, RESEND.

C	GET THE SIZE OF THE DATA AND THE CHECKSUM (4 AND 3 BYTES RESPECTIVELY)
	DECODE (7,9901,RBUFFER(IDX),ERR=30) DSIZE, CHECKSUM
9901	FORMAT (I4,I3)

	IF (DSIZE .LT. 0) GO TO 100	! IF < 0, END OF FILE.

C	NOW CHECK THE FOLLOWING THINGS:  MAKE SURE LENGTH OF RECORD IS RIGHT,
C	THAT THE LENGTH OF A BINARY TRANSFER IS A MULTIPLE OF 8, AND THAT
C	THE CHECKSUM CHECKS.
	IF (NBYTES .NE. DSIZE+7 .OR.
	1	(MODE .EQ. SYSGEN .AND. MOD(DSIZE,8) .NE. 0) .OR.
	1	(GET_CHECKSUM(DSIZE,RBUFFER(IDX+7)) .NE. CHECKSUM))
	1		GO TO 30

C	IF HERE, EVERYTHING IS OK.  WRITE THE RECORD TO THE OUTPUT FILE IN
C	THE CASE OF ASCII TRANSFERS, OTHERWISE CALL THE DECODING ROUTINE.
	LINNUM = LINNUM+1		! INCREMENT LINE NUMBER
	ACK = 1HY			! AND SET ACK TO "Y" (OK)
	IF (MODE .EQ. ASCII .OR. MODE .EQ. LISTING) THEN
		IF (DSIZE .GT. 0) THEN
			WRITE (1,9903) (RBUFFER(I+IDX+6),I=1,DSIZE)
9903			FORMAT (<DSIZE>A1)
		ELSE
			WRITE (1,9908)
9908			FORMAT ()
		ENDIF
	ELSE ! MODE .EQ. SYSGEN
		CALL DECODE_BIN( RBUFFER(IDX+7), DSIZE )
	ENDIF

30	IF (ACK .EQ. 1HY) THEN
		CALL DOYES(DSIZE)
	ELSE
		CALL DONO
	ENDIF
	CALL SENDACK(ACK)		! SEND EITHER ACK OR NAK TO REMOTE.
	GO TO 10			! LOOP BACK FOR NEXT LINE IF ANY.

100	ACK = 1HY
	CALL SENDACK(ACK)		! SEND ACK.
	IF (MODE .EQ. SYSGEN) THEN
		BCOUNT = BCOUNT / 2	! BYTE COUNT IS HALF # OF NIBBLES.
		! EMPTY THE BUFFER.
		IF (BUFIDX .NE. 0) WRITE (1) (VAXBUF(I),I=1,BUFIDX)
	ENDIF

	GET = .TRUE.
	RETURN
	END
	SUBROUTINE DECODE_BIN( TENBUF, NBYTES )
C
C	DECODES AN ASCII RECORD REPRESENTING A BINARY RECORD FROM THE
C	PDP-10.
C
	INCLUDE 'COM.INC/NOLIST'
C
	BYTE TENBUF(1)
	INTEGER NBYTES
C
	INTEGER*4 ITMP			! BECAUSE OF INT OVERFLOWS ON CVTLB
	BYTE	  ITMPBYTE		! AND ON MULB BECAUSE OF SIGN
	EQUIVALENCE (ITMP,ITMPBYTE)	! EXTENSION PROBLEMS.
C
C	DO SOME LOOP UNROLLING HERE FOR PROCESSING SPEED.  SINCE WE KNOW THAT
C	THE FILE WILL BE AN INTEGRAL NUMBER OF TOPS-10 WORDS LONG, WE AVOID
C	LOOP OVERHEAD; SINCE WE KNOW VAXBUF IS AN INTEGRAL NUMBER OF VAX-11
C	WORDS LONG, WE AVOID EXCESSIVE BUFFER FULL CHECKING.
C
	DO 10 I=0,NBYTES-8,8		! INCR 8 BYTES (NIBBLES) AT A TIME.
	IF (BUFIDX .EQ. 4*UYK_REC_SIZE) THEN ! WRITE OUT THE BUFFER IF FULL
		WRITE (1) VAXBUF
		BUFIDX = 0
	ENDIF

C	NOW DECODE THE NIBBLES INTO THE BUFFER
	DO 20 J=2,8,2
	ITMP = TENBUF(I+J-1) .AND. "77  ! CAUSE OF INT OVERFLOW ON 16* IF BYTE
	ITMP = 16*ITMP + (TENBUF(I+J) .AND. "77)
20	VAXBUF(BUFIDX+J/2) = ITMPBYTE	! SEE NOTE IN DECLARATION ABOVE.

C	FINALLY UP BUFFER POINTER.
10	BUFIDX = BUFIDX+4		! DO NEXT GROUP OF NIBBLES.
	RETURN				! AND RETURN WHEN DONE.
	END
	LOGICAL FUNCTION SEND
C
	INCLUDE 'COM.INC/NOLIST'
C
	INTEGER GET_CHECKSUM, WRIT10

	IF (MODE .EQ. ASCII .OR. MODE .EQ. TECO) THEN
		OPEN (UNIT=1,TYPE='OLD',NAME=VAXFILE(1:VSIZE),ERR=200)
	ELSE ! MODE .EQ. SYSGEN
		OPEN (UNIT=1, TYPE='OLD', NAME=VAXFILE(1:VSIZE),
	1		FORM='UNFORMATTED', RECORDTYPE='VARIABLE',
	1		RECORDSIZE=UYK_REC_SIZE, ERR=200)
	ENDIF

C
C	LOOP WRITING TO THE REMOTE CPU.
C
	LINNUM = 0
	BCOUNT = 0
	SEND = .FALSE.		! INIT RETURN VALUE TO ABORT.
10	IF (.NOT. INCMD) RETURN
	IF (MODE .EQ. ASCII) THEN
		READ (1,9901,END=100) NBYTES,(XBUFFER(I+7),I=1,NBYTES)
	ELSEIF (MODE .EQ. TECO) THEN
		READ (1,9901,END=400) NBYTES,(XBUFFER(I+1),I=1,NBYTES)
9901		FORMAT (Q,<NBYTES+1>A1)
	ELSE ! MODE .EQ. SYSGEN
		IF (.NOT. ENCODE_BIN (XBUFFER(8), NBYTES)) GO TO 100
	ENDIF

C	TECO INPUT USES NO PROTOCOL
	IF(MODE .EQ. TECO) GO TO 300

C	PROTOCOL IS USED HERE
C	WRITE TO THE REMOTE AND GET ACK/NAK BACK.
20	IF (.NOT. INCMD) RETURN
	IRCODE = WRIT10(NBYTES,GET_CHECKSUM(NBYTES,XBUFFER(8)),NBYTES+7)
	IF (IRCODE .EQ. %LOC(SS$_CANCEL)) RETURN
	I = RIOSB(2)
	CALL STRIPLF (IDX, RBUFFER, I)
	IF (I .NE. 1) GO TO 40
	IF (RBUFFER(IDX) .EQ. 1HY) THEN
		LINNUM = LINNUM + 1
		CALL DOYES(NBYTES)
		GO TO 10			! GO DO NEXT RECORD.
	ELSEIF (RBUFFER(IDX) .EQ. 1HN) THEN
		CALL DONO
		GO TO 20
	ENDIF
40	TYPE *,'REMOTE CPU RETURNED NEITHER Y NOR N FOR CONFIRMATION.'
	RETURN

C	HERE FOR EOF ON INPUT.
100	IRCODE = WRIT10( -1, -1, 7 )
	IF (IRCODE .EQ. %LOC(SS$_CANCEL)) RETURN
	I = RIOSB(2)
	CALL STRIPLF (IDX, RBUFFER, I)
	IF (I .NE. 1) GO TO 110
	IF (RBUFFER(IDX) .EQ. 1HN) THEN
		TYPE *,'TRANSMISSION ERROR ON EOF MARKER: RETRYING.'
		CALL WAITABIT('1')
		GO TO 100
	ELSEIF (RBUFFER(IDX) .EQ. 1HY) THEN
		SEND = .TRUE.
		RETURN
	ENDIF
110	TYPE *,'REMOTE CPU RETURNED NEITHER Y NOR N FOR EOF CONFIRMATION.'
	RETURN

200	TYPE *,'ERROR OPENING INPUT FILE.'
	RETURN

C	SEND TECO INPUT LINE AND WAIT FOR ANY REPLY
300	IF(.NOT. INCMD) RETURN
	CALL CLRTYPE
	XBUFFER(1)=1HI	!TECO INSERT COMMAND
	XBUFFER(NBYTES+2)="15	!CARRIAGE RETURN
	XBUFFER(NBYTES+3)="33	!ESCAPE
	XBUFFER(NBYTES+4)="33	!ESCAPE

	IRCODE=SYS$QIOW(,%VAL(RCHANOUT),%VAL(%LOC(IO$_WRITEPBLK)),
	1   XIOSB,,,XBUFFER,%VAL(NBYTES+4),,,,)	!SEND LINE TO TECO
	IF(IRCODE .NE. %LOC(SS$_NORMAL)) RETURN
	IRCODE=SYS$QIOW(,%VAL(RCHANIN),%VAL(%LOC(IO$_READPBLK)
	1		+%LOC(IO$M_TRMNOECHO)),
	1   RIOSB,,,RBUFFER,%VAL(1),,,,) !TECO MUST PROMPT
	IF(IRCODE .NE. %LOC(SS$_NORMAL)) RETURN
	CALL WAITABIT('1')  !WAIT LETS ALL ECHOING COME TO REST

	LINNUM = LINNUM + 1
	CALL DOYES(NBYTES)
	GO TO 10

C	END OF TECO INPUT - CLOSE FILE AND EXIT WITH EX$$
400	IF(.NOT. INCMD) RETURN
	XBUFFER(1)=1HE
	XBUFFER(2)=1HX
	XBUFFER(3)="33
	XBUFFER(4)="33
	CALL CLRTYPE

	IRCODE=SYS$QIOW(,%VAL(RCHANOUT),%VAL(%LOC(IO$_WRITEPBLK)),
	1   XIOSB,,,XBUFFER,%VAL(4),,,,)	!SEND EX$$
	IF(IRCODE .EQ. %LOC(SS$_CANCEL)) RETURN

	IRCODE=SYS$QIOW(,%VAL(RCHANIN),%VAL(%LOC(IO$_READPBLK)
	1		+%LOC(IO$M_TRMNOECHO)),
	1   RIOSB,,,RBUFFER,%VAL(1),,,,)	!TECO MUST PROMPT
	IF(IRCODE .EQ. %LOC(SS$_CANCEL)) RETURN
	CALL WAITABIT('1')

	IF(IRCODE .NE. %LOC(SS$_NORMAL)) THEN
		TYPE *, 'TRANSMISSION ERROR ON TECO EX$$:  RETRYING'
		GO TO 400
	ELSE
		TYPE *, 'TECO TRANSMISSION COMPLETED.  FILE CLOSED.'
		CALL CLRTYPE
		SEND = .TRUE.
		RETURN
	ENDIF
	END
	INTEGER FUNCTION WRIT10(NBYTES, CHECKSUM, BUFLEN)
C
C	PUTS THE LENGTH AND CHECKSUM INFORMATION INTO XBUFFER AND CALLS
C	WRTRD TO WRITE THE BUFFER OUT.
C	RETURNS THE STATUS CODE OF THE LAST SYSTEM SERVICE EXECUTED.
C
	INCLUDE 'COM.INC/NOLIST'
C
	INTEGER NBYTES, CHECKSUM, BUFLEN
	INTEGER WRTRD
C
	ENCODE (7,1,XBUFFER(1)) NBYTES, CHECKSUM
1	FORMAT (I4,I3)
	WRIT10 = WRTRD(BUFLEN)
	RETURN
	END
	LOGICAL FUNCTION INIT10
C
C	THIS FUNCTION PERFORMS THE INITIAL HANDSHAKE WITH THE RUNNING SNDRCV
C	PROGRAM, BY SENDING IT THE DIRECTION AND MODE OF THE FILE TRANSFER,
C	AND THE NAME OF THE REMOTE CPU'S FILE.
C
	INCLUDE 'COM.INC/NOLIST'
C
	INTEGER WRTRD, TSIZE
	LOGICAL TRY10
	BYTE TOVAX(5), TO10(5), ASC(6), SYSGN(6)
	DATA TOVAX/1HT,1HO,1HV,1HA,1HX/, TO10/1HT,1HO,1H1,1H0,1H /
	DATA ASC/1HA,1HS,1HC,1HI,1HI,1H /, SYSGN/1HS,1HY,1HS,1HG,1HE,1HN/
C
	INIT10 = .FALSE.		! SET VALUE OF FUNCTION TO FALSE.

C	PREPARE TO SEND THE DIRECTION.
	IF (FLOW .EQ. IN) THEN
		DO 1 I=1,5
1		XBUFFER(I) = TOVAX(I)
	ELSE
		DO 2 I=1,5
2		XBUFFER(I) = TO10(I)
	ENDIF
	IF (.NOT. TRY10(5)) GO TO 100

C	PREPARE TO SEND THE MODE NAME.
	IF (MODE .EQ. ASCII .OR. MODE .EQ. LISTING) THEN
		DO 3 I=1,6
3		XBUFFER(I) = ASC(I)
	ELSE
		DO 4 I=1,6
4		XBUFFER(I) = SYSGN(I)
	ENDIF
	IF (.NOT. TRY10(6)) GO TO 100

C	WE ESTABLISHED COMMUNICATION CORRECTLY.  NOW OPEN THE -10 FILE.
	WRITE (6,9905)
9905	FORMAT(' WHAT IS THE NAME OF THE REMOTE CPU''S FILE? '$)
	READ (5,9906) TSIZE,(XBUFFER(I),I=1,TSIZE)
9906	FORMAT (Q,<TSIZE+1>A1)

	IRCODE = WRTRD(TSIZE)
	IF (IRCODE .NE. %LOC(SS$_NORMAL)) CALL QIOERROR(11,IRCODE)
	NBYTES = RIOSB(2)
	CALL STRIPLF(IDX, RBUFFER, NBYTES)
	IF (NBYTES .EQ. 1 .AND. RBUFFER(IDX) .EQ. 1HY) THEN
		INIT10 = .TRUE.
		RETURN
	ENDIF

	! ERROR
100	TYPE 101,(RBUFFER(I),I=IDX,NBYTES+IDX-1)
101	FORMAT(1X,<NBYTES+1>A1)
	RETURN
	END
	LOGICAL FUNCTION TRY10(SIZE)
C
C	TRIES 10 TIMES TO INITIATE COMMUNICATION WITH THE REMOTE CPU.
C
	INCLUDE 'COM.INC/NOLIST'
	INTEGER SIZE, RETRIES, WRTRD
C
	TRY10 = .TRUE.
	DO 1 RETRIES = 1,10		! TRY 10 TIMES.
	IRCODE = WRTRD(SIZE)
	IF (IRCODE .NE. %LOC(SS$_NORMAL)) GO TO 1

	! CHECK FOR "Y" OR "N"
	NBYTES = RIOSB(2)
	CALL STRIPLF(IDX, RBUFFER, NBYTES) ! STRIP LEADING LF IF ANY.
	IF (NBYTES .NE. 1) GO TO 4
	IF (RBUFFER(IDX) .EQ. 1HY) RETURN
	IF (RBUFFER(IDX) .NE. 1HN) GO TO 4
1	CONTINUE
	TRY10 = .FALSE.
	RETURN				! RETURN IN DISGRACE.

4	TYPE *,'THE REMOTE CPU RETURNED NEITHER Y NOR N.'
	TRY10 = .FALSE.
	RETURN
	END
	SUBROUTINE CLRTYPE
C
C	CLEARS THE TYPEAHEAD BUFFER ON THE REMOTE CHANNEL.
C
	INCLUDE 'COM.INC/NOLIST'
C
	IRCODE = SYS$QIOW(,%VAL(RCHANIN),
	1	%VAL(%LOC(IO$_READPBLK)+%LOC(IO$M_PURGE)),
	2	RIOSB,,,RBUFFER,%VAL(0),,,,)
	RETURN
	END
	SUBROUTINE DOYES(NBYTES)
C
C	UPDATES BCOUNT BY THE NUMBER OF BYTES AND PRINTS A MESSAGE IF
C	NEEDED.
C
	INCLUDE 'COM.INC/NOLIST'
C
	INTEGER NBYTES
C
	BCOUNT = BCOUNT + NBYTES
	IF (0 .EQ. MOD(LINNUM,10)) TYPE 1,LINNUM-9,LINNUM
1	FORMAT(' RECORDS ',I7,' THROUGH ',I7,' SUCCESSFULLY'
	1		' TRANSFERRED.')
	RETURN

	ENTRY DONO
	TYPE 2,LINNUM+1
2	FORMAT (' TRANSMISSION ERROR ON RECORD ',I7,': RETRYING.')
	CALL WAITABIT('1')
	RETURN

	ENTRY PRTFINAL		! WRITE OUT THE FINAL MESSAGE.
	IF     (0 .EQ. MOD(LINNUM,10)) THEN
		RETURN
	ELSEIF (1 .EQ. MOD(LINNUM,10)) THEN
		TYPE 3, LINNUM
3		FORMAT(' RECORD  ',I7,' SUCCESSFULLY TRANSFERRED.')
	ELSE
		TYPE 1, LINNUM+1-MOD(LINNUM,10), LINNUM
	ENDIF
	RETURN
	END
	INTEGER FUNCTION GET_CHECKSUM(LEN,BUFF)
C
C	COMPUTES THE CHECKSUM.
C
	INTEGER LEN
	BYTE BUFF(1)

	GET_CHECKSUM = 0
	IF (LEN .GT. 0) THEN
		DO 10 I=1,LEN
10		GET_CHECKSUM = GET_CHECKSUM + BUFF(I)
		GET_CHECKSUM = GET_CHECKSUM .AND. "777
	ENDIF
	RETURN
	END
	SUBROUTINE SENDACK(CODE)
C
C	SENDS AN ACK/NAK CODE BACK TO THE REMOTE CPU.
C
	INCLUDE 'COM.INC/NOLIST'
C
	BYTE CODE
C
	CALL CLRTYPE	!CLEAR TYPE AHEAD BUFFER ON REMOTE CHANNEL
	RBUFFER(1) = CODE
	RBUFFER(2) = "15
	I = SYS$QIOW(,%VAL(RCHANOUT),%VAL(%LOC(IO$_WRITEPBLK)),
	1		RIOSB,,,RBUFFER,%VAL(2),,,,)
	RETURN
	END
	SUBROUTINE STRIPLF(IDX, BUFF, NBYTES)
C
C	DETERMINES IF THE FIRST CHARACTER IN THE BUFFER IS A LINE FEED.
C	IF NOT, SETS IDX TO 1 (BUFFER INDEX) AND RETURNS.
C	IF SO, DECREMENTS NBYTES (BYTE COUNT), SETS IDX TO 2, AND RETURNS.
C
	BYTE BUFF(1)
	INTEGER IDX, NBYTES

	IDX = 0
1	IDX = IDX+1
	IF (BUFF(IDX) .EQ. "12) GO TO 1

	NBYTES = NBYTES-IDX+1
	RETURN
	END
	INTEGER FUNCTION WRTRD(LEN)
C
C	APPEND A <CR> TO XBUFFER AND WRITE IT OUT, WITH LENGTH LEN.
C	THEN READ THE REMOTE CPU'S RESPONSE INTO RBUFFER.
C
	INCLUDE 'COM.INC/NOLIST'
	INTEGER LEN
C
	XBUFFER(LEN+1) = "15
	CALL CLRTYPE
	WRTRD = SYS$QIOW(,%VAL(RCHANOUT),%VAL(%LOC(IO$_WRITEPBLK)),
	1		XIOSB,,,XBUFFER,%VAL(LEN+1),,,,)
	IF (WRTRD .NE. %LOC(SS$_NORMAL)) RETURN

C	RECEIVE THE ACK/NAK BACK FROM THE REMOTE CPU.
100	IF(.NOT. INCMD) RETURN
	WRTRD = SYS$QIOW(,%VAL(RCHANIN),%VAL(%LOC(IO$_READPBLK)
	1		+%LOC(IO$M_TRMNOECHO)),
	1		RIOSB,,,RBUFFER,%VAL(1),,,,)
	IF(RBUFFER(1).LE."40) GO TO 100
	IF(RBUFFER(1).EQ."177) GO TO 100
	RETURN
	END
	SUBROUTINE WAITABIT(SECONDS)
C	THIS SUBROUTINE JUST WAITS A LITTLE THEN RETURNS
	CHARACTER*(*) SECONDS
	INTEGER*4 DELTA
	CALL SYS$BINTIM('0 00:00:'//SECONDS,DELTA)
	CALL SYS$SCHDWK(,,DELTA,,)
	CALL SYS$HIBER
	RETURN
	END
